#include "FastLED.h"


#define NUM_LEDS 90
#define DATA_PIN 5

#define player1_button 14 // A0
#define player2_button 17 //  A3

#define GS_DEMO    0
#define GS_SERVING 1
#define GS_ROLLING 2
#define GS_LOSING  3
#define GS_WINNING_POINT 4
#define GS_WINNING_MATCH 5

#define DEMO_COLOR CRGB::Green
#define DEMO_SPEED 10
#define SERVE_SPEED 30

#define MAX_SCORE 5

CRGB leds[NUM_LEDS];

struct player
{
  unsigned char score;
  unsigned char end_start, end_stop, serve_start, serve_stop, target_pos;
  char serve_direction;
  unsigned char click_processed;
  CRGB color, win1, win2;
} players[2];

unsigned int player_button[2];


char clicked()
{
  for (int i = 0; i < 2; i++)
  {   
    if (digitalRead(player_button[i]) == 1 && (players[i].click_processed == 1))
    {
      players[i].click_processed = 0;
      return -1;
    }
    
    if (digitalRead(player_button[i]) == 0 && (players[i].click_processed == 0))
    {      
      return i;
    }

    
  }
  return -1;

}


void light_off()
{
  for (int k = 0; k < NUM_LEDS; k++ )
  {
    leds[k] = CRGB::Black;
  }
  FastLED.show();
}


void setup() {
  Serial.begin(115200);
  delay(200);

  player_button[0] = player1_button;
  player_button[1] = player2_button;

  pinMode(player_button[0], INPUT_PULLUP);
  pinMode(player_button[1], INPUT_PULLUP);

   

  // hack to use pins A1, A4 as GND and A2, A5 as 5V
  pinMode(15, OUTPUT);
  pinMode(16, OUTPUT);
  pinMode(18, OUTPUT);
  pinMode(19, OUTPUT);

  digitalWrite(15, LOW);
  digitalWrite(16, HIGH);

  digitalWrite(18, LOW);
  digitalWrite(19, HIGH);
  

  Serial.print("Start ");
  Serial.print(player_button[0]);
  Serial.print(" - ");
  Serial.println(player_button[1]);
  
  
  FastLED.addLeds<WS2811, DATA_PIN, GRB>(leds, NUM_LEDS);
  pinMode(player1_button, INPUT);
  pinMode(player2_button, INPUT);
  pinMode(13, OUTPUT);

  light_off();

  players[0].score = 0;
  players[0].end_start = 0;
  players[0].end_stop = 9;
  players[0].serve_start = 0;
  players[0].serve_stop = 9;
  players[0].target_pos = NUM_LEDS - 1;
  players[0].serve_direction = 1;
  players[0].click_processed = 0;
  players[0].color = CRGB::Red;
  players[0].win1 = CRGB::Black;
  players[0].win2 = CRGB::Red;
  
  
  players[1].score = 0;
  players[1].end_start = NUM_LEDS - 11;
  players[1].end_stop = NUM_LEDS - 1;
  players[1].serve_start = NUM_LEDS - 1;
  players[1].serve_stop = NUM_LEDS - 11;
  players[1].target_pos = 0;
  players[1].serve_direction = -1;
  players[1].click_processed = 0;
  players[1].color = CRGB::Green;
  players[1].win1 = CRGB::Black;
  players[1].win2 = CRGB::Green;


}


void demo()
{
  unsigned int ball_position = 0;
  char ball_direction = 1;
  char active_player;
  CRGB ball_color = DEMO_COLOR;

  while (true)
  {
    if ( (ball_position == (NUM_LEDS -1)) && (ball_direction == 1) )
      ball_direction = -1;
      
    if ( (ball_position == 0) && (ball_direction == -1) )
      ball_direction = 1;

    leds[ball_position] = ball_color;
    FastLED.show();
    delay(DEMO_SPEED);
    leds[ball_position] = CRGB::Black;
    ball_position += ball_direction;

    active_player = clicked();

    if (active_player != -1)
    {
      ball_color = players[serve(active_player)].color;
    }
  }
  
  
}

// returns game winner
unsigned char serve(unsigned char start_player)
{ 
  unsigned char active_player, other_player, click_player;
  unsigned int ball_position = 0;
  char ball_direction = 1;
  unsigned int serve_speed;
  unsigned char winner;
  
  active_player = start_player;

  players[active_player].click_processed = 1;

  if (active_player == 0)
  {
    other_player = 1;
  }
  else
  {
    other_player = 0;
  }

  ball_position = players[active_player].serve_start;
  ball_direction = players[active_player].serve_direction;

  while (players[0].score < MAX_SCORE && players[1].score < MAX_SCORE)
  {
    leds[9] = players[0].color;
    leds[79] = players[1].color;

    switch (active_player)
    {
      case 0:
        if ( (ball_position == players[active_player].serve_start) && (ball_direction == -1) )
        {
          ball_direction = 1;
        }

        if ( (ball_position == players[active_player].serve_stop) && (ball_direction == 1) )
        {
          ball_direction = -1;
        }
        break;

        case 1:
        if ( (ball_position == players[active_player].serve_stop) && (ball_direction == -1) )
        {
          ball_direction = 1;
        }

        if ( (ball_position == players[active_player].serve_start) && (ball_direction == 1) )
        {
          ball_direction = -1;
        }
        break;
    }

    leds[ball_position] = players[active_player].color;
    FastLED.show();
    delay(SERVE_SPEED);
    leds[ball_position] = CRGB::Black;
    ball_position += ball_direction;

    click_player = clicked();

    if (click_player == active_player)
    {
      switch (active_player)
      {
        case 0:
          serve_speed = ball_position * 2;
          break;
       case 1:
          serve_speed = (players[active_player].serve_start - ball_position) * 2;
          break;
      }
      
      
      roll(active_player, ball_position, serve_speed);

      active_player = other_player;
      
      if (active_player == 0)
      {
        other_player = 1;
      }
      else
      {
        other_player = 0;
      }
    
      ball_position = players[active_player].serve_start;
      ball_direction = players[active_player].serve_direction;
    }
    
  }

  winner = 1;
  if (players[0].score == MAX_SCORE)
    winner = 0;

  players[0].score = 0;
  players[1].score = 0;

  return winner;
  
}



void roll(unsigned char start_player, unsigned char serve_start, unsigned int serve_speed)
{
  unsigned char active_player, other_player, click_player;
  unsigned int ball_position = 0;
  char ball_direction = 1;
  unsigned int ball_speed = serve_speed;
  active_player = start_player;

  players[active_player].click_processed = 1;

  if (active_player == 0)
  {
    other_player = 1;
  }
  else
  {
    other_player = 0;
  }

  ball_position = serve_start;
  ball_direction = players[active_player].serve_direction;

  while (true)
  {
    leds[9] = players[0].color;
    leds[79] = players[1].color;  

    switch (active_player)
    {
      case 0:
        if ( (ball_position == (NUM_LEDS - 1)) && (ball_direction == 1) )
        {
          increase_score(0, ball_position);
          return;
        }
        
        break;

        case 1:
        if ( (ball_position == 0) && (ball_direction == -1) )
        {
          increase_score(1, ball_position);
          return;
        }
        
        break;
    }

    leds[ball_position] = players[active_player].color;
    FastLED.show();
    delay(ball_speed);
    leds[ball_position] = CRGB::Black;
    ball_position += ball_direction;

    click_player = clicked();

    if (click_player == active_player)
    {
      players[click_player].click_processed = 1;
      increase_score(other_player, ball_position);
      return;      
    }

    if (click_player == other_player)
    {
      players[click_player].click_processed = 1;

      // opposite player clicked in his serve zone
      if ( (players[click_player].end_start <= ball_position) && (ball_position <= players[click_player].end_stop) )
      {
        active_player = other_player;
        if (active_player == 0)
        {
          other_player = 1;
        }
        else
        {
          other_player = 0;
        }
  
        if (ball_direction == 1)
        {
          ball_direction = -1;
        }
        else
        {
          ball_direction = 1;
        }

        switch (active_player)
        {
          case 0:
            ball_speed = ball_position * 2;
            break;
         case 1:
            ball_speed = (players[active_player].serve_start - ball_position) * 2;
            break;
        }
        
      }
      else
      {
        increase_score(active_player, ball_position);
        return;      
      }
      
      
      
    }
    
  }

  
}

void increase_score(unsigned char player, unsigned char ball_position)
{
  leds[ball_position] = CRGB::Black;
  FastLED.show();
  
  players[player].score++;
  for (int i = 0; i < players[player].score; i++)
  {
    if (players[player].score == MAX_SCORE)
    {
      for (int i = 0; i < NUM_LEDS; i++)
      {
        leds[i] = players[player].win1;
      }
    }
    else
    {
      leds[ball_position] = players[player].win1;
    }
    
    FastLED.show();
    delay(400);
    if (players[player].score == MAX_SCORE)
    {
      for (int i = 0; i < NUM_LEDS; i++)
      {
        leds[i] = players[player].win2;
      }
    }
    else
    {
      leds[ball_position] = players[player].win2;
    }
    FastLED.show();
    delay(400);
  }

  leds[ball_position] = CRGB::Black;
  FastLED.show();
  
}


void loop() 
{
  demo();
}


